Package: periscope2
Type: Package
Title: Enterprise Streamlined 'shiny' Application Framework Using
        'bs4Dash'
Version: 0.4.0
Authors@R: c(
    person("Mohammed", "Ali",      role = c("aut", "cre"), email   = "mohammed@aggregate-genius.com"),
    person("Constance", "Brett",   role = c("ctb")),
    person("Aggregate Genius Inc", role = c("spn"))
    )
Description: A framework for building enterprise, scalable and UI-standardized 'shiny' applications.  
    It brings enhanced features such as 'bootstrap' v4 <https://getbootstrap.com/docs/4.0/getting-started/introduction/>,
    additional and enhanced 'shiny' modules, customizable UI features, as well as an enhanced application file
    organization paradigm. This update allows developers to harness the ability to build powerful applications and 
    enriches the 'shiny' developers' experience when building and  maintaining applications.
URL: https://github.com/Aggregate-Genius/periscope2,
        http://periscopeapps.org:3838
BugReports: https://github.com/Aggregate-Genius/periscope2/issues
License: GPL-3
Encoding: UTF-8
Language: en-US
Depends: R (>= 4.2)
Imports: shiny (>= 1.7), bs4Dash (>= 2.3), DT, fresh, grDevices,
        lubridate, methods, reactable, shinyFeedback, shinyWidgets,
        utils, writexl, yaml, lifecycle
Suggests: assertthat, canvasXpress, colourpicker, ggplot2, knitr,
        lattice, miniUI, openxlsx, openxlsx2, rmarkdown, shinyjs,
        spelling, testthat, waiter
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
KeepSource: no
NeedsCompilation: no
Packaged: 2026-08-19 13:11:53 UTC; mohammedali
Author: Mohammed Ali [aut, cre],
  Constance Brett [ctb],
  Aggregate Genius Inc [spn]
Maintainer: Mohammed Ali <mohammed@aggregate-genius.com>
Repository: CRAN
Date/Publication: 2026-08-19 16:10:02 UTC
