# -------------------------------------------------
# --               FRAMEWORK ui.R                --
# -------------------------------------------------
# NOTES:                                         --
# Program code goes in "program" directory files --
# -------------------------------------------------
# *****        DO NOT EDIT THIS FILE       ***** --
# -------------------------------------------------

library(bs4Dash)

source(paste("program", "ui_body.R", sep = .Platform$file.sep), local = TRUE)
source(paste("program", "ui_header.R", sep = .Platform$file.sep), local = TRUE)

source(paste("program", "ui_right_sidebar.R", sep = .Platform$file.sep), local = TRUE)
source(paste("program", "ui_footer.R", sep = .Platform$file.sep), local = TRUE)
periscope2:::create_application_dashboard()
