# logViewerOutput

    [[1]]
    <div class="shiny-panel-conditional" data-display-if="output.displayButton" data-ns-prefix="myid-myid-">
      <span id="myid-myid-reactTableButtonDiv" class="periscope-downloadable-react-table-button" style="">
        <span class="btn-group" data-toggle="tooltip" data-placement="top" title="Download application logs">
          <button aria-expanded="false" aria-haspopup="true" class="btn btn-default action-button dropdown-toggle periscope-download-btn" data-toggle="dropdown" id="myid-myid-reactTableButtonID-downloadFileList" type="button action">
            <i class="far fa-copy" role="presentation" aria-label="copy icon"></i>
          </button>
          <ul class="dropdown-menu" id="myid-myid-reactTableButtonID-testList">
            <li>
              <a aria-disabled="true" class="shiny-download-link disabled periscope-download-choice" download href="" id="myid-myid-reactTableButtonID-csv" tabindex="-1" target="_blank">csv</a>
            </li>
            <li>
              <a aria-disabled="true" class="shiny-download-link disabled periscope-download-choice" download href="" id="myid-myid-reactTableButtonID-tsv" tabindex="-1" target="_blank">tsv</a>
            </li>
          </ul>
        </span>
      </span>
    </div>
    
    [[2]]
    <div class="reactable html-widget html-widget-output shiny-report-size html-fill-item" data-reactable-output="myid-myid-reactTableOutputID" id="myid-myid-reactTableOutputID" style="width:auto;height:auto;"></div>
    

