# theme builder addin - server

    Code
      yaml::read_yaml(output$downloadConfig)
    Output
      $primary
      [1] "#B221DD"
      
      $secondary
      [1] "#6C757D"
      
      $success
      [1] "#2ED610"
      
      $info
      [1] "#7BDFF2"
      
      $warning
      [1] "#FFF200"
      
      $danger
      [1] "#CE0900"
      
      $light
      [1] "#F8F9FA"
      
      $dark
      [1] "#343A40"
      
      $bg
      [1] "#D67E7CD1"
      
      $hover_bg
      [1] "#B0B818F7"
      
      $color
      [1] "#CA39D4ED"
      
      $hover_color
      [1] "#000000E4"
      
      $active_color
      [1] "#BF8DF0EE"
      
      $submenu_bg
      [1] "#084D66BF"
      
      $submenu_color
      [1] "#B33634D3"
      
      $submenu_hover_color
      [1] "#0303038F"
      
      $submenu_hover_bg
      [1] "#BE25CFF2"
      
      $submenu_active_color
      [1] "#1FAB9DC2"
      
      $submenu_active_bg
      [1] "#401487E8"
      
      $header_color
      [1] "#D91F1CAF"
      
      $sidebar_width
      [1] "350"
      
      $control_sidebar_width
      [1] "250"
      
      $sidebar_padding_x
      [1] "15"
      
      $sidebar_padding_y
      [1] "20"
      
      $sidebar_mini_width
      [1] "25"
      
      $blue
      [1] "#007BFF"
      
      $lightblue
      [1] "#3C8DBC"
      
      $navy
      [1] "#001F3F"
      
      $cyan
      [1] "#17A2B8"
      
      $teal
      [1] "#39CCCC"
      
      $olive
      [1] "#3D9970"
      
      $green
      [1] "#28A745"
      
      $lime
      [1] "#01FF70"
      
      $orange
      [1] "#FF851B"
      
      $yellow
      [1] "#FFC107"
      
      $fuchsia
      [1] "#F012BE"
      
      $purple
      [1] "#605CA8"
      
      $maroon
      [1] "#D81B60"
      
      $red
      [1] "#DC3545"
      
      $black
      [1] "#111111"
      
      $gray_x_light
      [1] "#D2D6DE"
      
      $gray_600
      [1] "#6C757D"
      
      $white
      [1] "#FFFFFF"
      
      $contrasted_threshold
      [1] "12"
      
      $text_dark
      [1] "#701110D0"
      
      $text_light
      [1] "#2F515EBA"
      
      $`navbar-light-color`
      [1] "#bec5cb"
      

